package es.ucm.fdi.grasia.faerie.container;

import java.net.URI;
import java.util.UUID;

import es.ucm.fdi.grasia.faerie.core.ContextElement;

/**
 * Represents the single context container of a certain environment. It provides the same methods than
 * {@link net.faerie.core.Context}, but the caller must specify its identity on each call.
 * 
 * <h4>{@code Contextobserver}s and request/release mechanism</h4>
 * 
 * When a client invokes the {@link #request(net.faerie.core.ContextElement, java.util.UUID)} method, the context
 * container does the following:
 * <ol>
 * <li>It search an existing element with the same {@code elementId}.</li>
 * <li>If the element exists:
 * <ol>
 * <li>Returns the existing element.</li>
 * </ol>
 * otherwise:
 * <ol>
 * <li>It adds the element to the context.</li>
 * <li>It calls the {@link net.faerie.core.ContextObserver#elementAdded(net.faerie.core.ContextEvent)} method of all
 * registered {@code ContextObserver}s, provided that the method
 * {@link net.faerie.core.ContextObserver#handles(net.faerie.core.ContextEvent)} for the element returns true.</li>
 * </ol>
 * </li>
 * <li>It binds the {@code elementId} to the {@code requesterId}.</li>
 * </ol>
 * 
 * When a client invokes the {@link #release(java.net.URI, java.util.UUID)} method, the context container does the
 * following:
 * <ol>
 * <li>It unbinds the {@code elementId} of the {@code requesterId}.</li>
 * <li>If the {@code elementId} has no bound {@code requesterId}s:
 * <ol>
 * <li>It calls the {@link net.faerie.core.ContextObserver#elementRemoved(net.faerie.core.ContextEvent)} method of all
 * registered {@code ContextObserver}s, provided that the method
 * {@link net.faerie.core.ContextObserver#handles(net.faerie.core.ContextEvent)} for the event returns true.</li>
 * <li>It removes the element from the context.</li>
 * </ol>
 * </li>
 * </ol>
 * 
 * @author Chema
 * @see ContextElement
 * @see net.faerie.core.Context
 * @see net.faerie.core.ContextManager
 * @see ContextContainerService
 * @see net.faerie.core.ContextObserver
 */
public interface ContextContainer {

	/**
	 * Test if the context contains the element with the specified {@code elementId}.
	 * 
	 * @param elementId the URI of the element.
	 * @param requesterId the requester UUID.
	 * @return true if the element is contained in the context, false otherwise.
	 */
	public boolean contains(URI elementId, UUID requesterId);

	/**
	 * Obtains the element with the specified {@code elementId} if it exists in the context. Returns {@code null}
	 * otherwise.
	 * 
	 * @param <T> the subclass of the element.
	 * @param elementId the URI of the element.
	 * @param requesterId the requester UUID.
	 * @return the element with the specified URI, if exists in the context, or null otherwise.
	 */
	public <T extends ContextElement> T peek(URI elementId, UUID requesterId);

	/**
	 * Requests the specified {@code element}.
	 * 
	 * @param <T> the subclass of the element.
	 * @param element the element requested.
	 * @param requesterId the requester UUID.
	 * @return the specified element, following the rules described at the beginning.
	 */
	public <T extends ContextElement> T request(T element, UUID requesterId);

	/**
	 * Releases the {@code ContextElement} with the specified {@code elementId}.
	 * 
	 * @param elementId the URI of the element.
	 * @param requesterId the requester UUID.
	 */
	public void release(URI elementId, UUID requesterId);

	/**
	 * Releases all the {@code ContextElement}s requested by the specified {@code requesterId}.
	 * 
	 * @param requesterId the requester UUID.
	 */
	public void releaseAll(UUID requesterId);
}
